import { Module, Provider } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import { AD_MESSAGE_PUBLISHER, AD_REPOSITORY } from './ad.di-tokens';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { AdRepository } from './infrastructure/ad.repository';
import { PrismaService } from './infrastructure/prisma.service';
import { AdMapper } from './ad.mapper';
import { AdCreatedMessageHandler } from './interface/message-handlers/ad-created.message-handler';

const messageHandlers = [AdCreatedMessageHandler];

const mappers: Provider[] = [AdMapper];

const repositories: Provider[] = [
  {
    provide: AD_REPOSITORY,
    useClass: AdRepository,
  },
];

const messagePublishers: Provider[] = [
  {
    provide: AD_MESSAGE_PUBLISHER,
    useExisting: MessageBrokerPublisher,
  },
];
const orms: Provider[] = [PrismaService];

@Module({
  imports: [CqrsModule],
  providers: [
    ...messageHandlers,
    ...mappers,
    ...repositories,
    ...messagePublishers,
    ...orms,
  ],
  exports: [PrismaService, AdMapper, AD_REPOSITORY],
})
export class AdModule {}
